unit MRUfiles;

{

 TMRUFiles v1.0
 by Kambiz R. Khojasteh

 email: khojasteh@www.dci.co.ir
 web: http://www.crosswinds.net/~khojasteh/

 This component is freeware and may be used in any software product.

 If you modify the source, please send a copy of the new source to me.

}

interface

uses
  {$IFDEF WIN32} Windows {$ELSE} WinTypes, WinProcs {$ENDIF}, Classes, Menus;

type
  TMRUClickEvent = procedure(Sender: TObject; FileName: String) of object;
  TMRUFiles = class(TComponent)
  private
    FFiles: TStringList;
    FMaxEntries: Byte;
    FMenu: TMenu;
    FItemsPos: Word;
    FNumbered: Boolean;
    MenuItemCount: Word;
    FOnClick: TMRUClickEvent;
    procedure SetMaxEntries(Value: Byte);
    procedure SetMenu(Value: TMenu);
    procedure SetItemsPos(Value: Word);
    procedure SetNumbered(Value: Boolean);
    procedure FilesChanged(Sender: TObject);
    procedure MenuItemClicked(Sender: TObject);
    procedure ClearMenuItems;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add(FileName: String);
    procedure Remove(FileName: String);
    procedure Rename(OldFileName, NewFileName: String);
    {$IFDEF WIN32}
    procedure LoadFromRegistry(Key: String);
    procedure SaveToRegistry(Key: String);
    {$ENDIF}
    procedure LoadFromIni(IniFileName, Section: String);
    procedure SaveToIni(IniFileName, Section: String);
    procedure LoadFromFile(FileName: String);
    procedure SaveToFile(FileName: String);
    property Files: TStringList read FFiles;
  published
    property Menu: TMenu read FMenu write SetMenu;
    property ItemsPos: Word read FItemsPos write SetItemsPos default 0;
    property MaxEntries: Byte read FMaxEntries write SetMaxEntries default 9;
    property Numbered: Boolean read FNumbered write SetNumbered default True;
    property OnClick: TMRUClickEvent read FOnClick write FOnClick;
  end;

procedure Register;

implementation

{$IFDEF WIN32}
  {$R *.D32}
{$ELSE}
  {$R *.D16}
{$ENDIF}

uses
  SysUtils, Forms, IniFiles {$IFDEF WIN32}, Registry {$ENDIF};

constructor TMRUFiles.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMenu := nil;
  FItemsPos := 0;
  FMaxEntries := 9;
  FNumbered := True;
  FFiles := TStringList.Create;
  Files.OnChange := FilesChanged;
end;

destructor TMRUFiles.Destroy;
begin
  Files.Free;
  inherited Destroy;
end;

procedure TMRUFiles.Add(FileName: String);
var
  I: Integer;
begin
  if FileName <> EmptyStr then
  begin
    Files.BeginUpdate;
    try
      for I := 0 to Files.Count-1 do
        if CompareText(Files[I], FileName) = 0 then
        begin
          Files.Delete(I);
          Break;
        end;
      while Files.Count >= MaxEntries do
        Files.Delete(MaxEntries-1);
      Files.Insert(0, FileName);
    finally
      Files.EndUpdate;
    end;
  end;
end;

procedure TMRUFiles.Remove(FileName: String);
var
  I: integer;
begin
  if FileName <> EmptyStr then
    for I := 0 to Files.Count-1 do
      if (CompareText(Files[I], FileName) = 0) then
      begin
        Files.Delete(I);
        Break;
      end;
end;

procedure TMRUFiles.Rename(OldFileName, NewFileName: String);
var
  I: integer;
begin
  if OldFileName <> NewFileName then
    if OldFileName = EmptyStr then
      Add(NewFileName)
    else if NewFileName = EmptyStr then
      Remove(OldFileName)
    else
      for I := 0 to Files.Count-1 do
        if (CompareText(Files[I], OldFileName) = 0) then
        begin
          Files[I] := NewFileName;
          Break;
        end;
end;

{$IFDEF WIN32}

procedure TMRUFiles.LoadFromRegistry(Key: String);
var
  I: Integer;
  FileName: String;
  Registry: TRegistry;
begin
  Files.BeginUpdate;
  Files.Clear;
  Registry := TRegistry.Create;
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    if Registry.OpenKeyReadOnly(Key) then
    begin
      for I := 1 to MaxEntries do
      begin
        if Registry.ValueExists(Format('File%d', [I])) then
        begin
          FileName := Registry.ReadString(Format('File%d', [I]));
          if (FileName <> EmptyStr) and (Files.IndexOf(FileName) < 0) then
            Files.Add(FileName);
        end;
      end;
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToRegistry(Key: String);
var
  I: Integer;
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    Registry.DeleteKey(Key);
    if Registry.OpenKey(Key, True) then
    begin
      for I := 1 to Files.Count do
        Registry.WriteString(Format('File%d', [I]), Files[I-1]);
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

{$ENDIF}

procedure TMRUFiles.LoadFromIni(IniFileName, Section: String);
var
  I: Integer;
  FileName: String;
  Ini: TIniFile;
begin
  Files.BeginUpdate;
  Files.Clear;
  Ini := TIniFile.Create(IniFileName);
  try
    for I := 1 to MaxEntries do
    begin
      FileName := Ini.ReadString(Section, Format('File%d', [I]), EmptyStr);
      if FileName = EmptyStr then
        Break
      else if Files.IndexOf(FileName) < 0 then
        Files.Add(FileName);
    end;
  finally
    Ini.Free;
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToIni(IniFileName, Section: String);
var
  I: Integer;
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IniFileName);
  try
    Ini.EraseSection(Section);
    for I := 1 to Files.Count do
      Ini.WriteString(Section, Format('File%d', [I]), Files[I-1]);
  finally
    Ini.Free;
  end;
end;

procedure TMRUFiles.LoadFromFile(FileName: String);
begin
  Files.BeginUpdate;
  Files.Clear;
  try
    Files.LoadFromFile(FileName);
  finally
    Files.EndUpdate;
  end;
end;

procedure TMRUFiles.SaveToFile(FileName: String);
begin
  Files.SaveToFile(FileName);
end;

procedure TMRUFiles.SetMaxEntries(Value: Byte);
begin
  if MaxEntries <> Value then
  begin
    FMaxEntries := Value;
    Files.BeginUpdate;
    try
      while Files.Count >= MaxEntries do
        Files.Delete(MaxEntries-1);
    finally
      Files.EndUpdate;
    end;
  end;
end;

procedure TMRUFiles.SetNumbered(Value: Boolean);
begin
  if Numbered <> Value then
  begin
    FNumbered := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.SetItemsPos(Value: Word);
begin
  if not (csLoading in ComponentState) then
    if Menu = nil then
      Value := 0
    else if (Value <> 0) and (Value >= Menu.Items.Count) then
      Value := Menu.Items.Count-1;
  if ItemsPos <> Value then
  begin
    ClearMenuItems;
    FItemsPos := Value;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.SetMenu(Value: TMenu);
begin
  if Menu <> Value then
  begin
    ClearMenuItems;
    FMenu := Value;
    if (Menu <> nil) and (ItemsPos <> 0) and (ItemsPos >= Menu.Items.Count) then
      FItemsPos := Menu.Items.Count-1;
    FilesChanged(Self);
  end;
end;

procedure TMRUFiles.ClearMenuItems;
var
  I: Integer;
begin
  if Menu <> nil then
  begin
    for I := 1 to MenuItemCount do
      Menu.Items.Delete(ItemsPos);
    MenuItemCount := 0;
  end;
end;

procedure TMRUFiles.FilesChanged(Sender: TObject);
var
  I: Integer;
  MenuItem: TMenuItem;
begin
  if Menu <> nil then
  begin
    ClearMenuItems;
    for I := Files.Count-1 downto 0 do
    begin
      MenuItem := TMenuItem.Create(Self);
      MenuItem.Tag := I;
      MenuItem.Hint := Format('Opens %s', [Files[I]]);
      if not Numbered then
        MenuItem.Caption := Format('%s', [Files[I]])
      else if I < 10 then
        MenuItem.Caption := Format('&%d %s', [I+1, Files[I]])
      else
        MenuItem.Caption := Format('  %s', [Files[I]]);
      MenuItem.OnClick := MenuItemClicked;
      Menu.Items.Insert(ItemsPos, MenuItem);
      Inc(MenuItemCount);
    end;
    if (ItemsPos > 0) and (Files.Count > 0) then
    begin
      MenuItem := TMenuItem.Create(Self);
      MenuItem.Caption := '-';
      Menu.Items.Insert(ItemsPos, MenuItem);
      Inc(MenuItemCount);
    end;
  end;
end;

procedure TMRUFiles.MenuItemClicked(Sender: TObject);
begin
  if Assigned(FOnClick) and (Sender is TMenuItem) then
    FOnClick(Self, Files[TMenuItem(Sender).Tag]);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TMRUFiles]);
end;

end.
